import { deepAssign } from '@codeperate/utils';
import { StringWidgetProps } from './widgets/StringWidget.config';
import { TextAreaWidgetProps } from './widgets/TextAreaWidget.config';
import { PasswordWidgetProps } from './widgets/PasswordWidget.config';
import { ArrayWidgetContext } from './widgets/ArrayWidget.config';
import { NumberWidgetProps } from './widgets/NumberWidget.config';
import { DateWidgetProps } from './widgets/DateWidget.config';
import { SectionWidgetProps } from './widgets/SectionWidget.config';
import { ObjectWidgetProps } from './widgets/ObjectWidget.config';
import { FileWidgetProp } from './widgets/FileWidget.config';
import { CheckBoxWidgetProps } from './widgets/CheckBoxWidget.config';

type ToDefault<T extends object> = {
    [K in keyof T]: T[K] extends object ? (T[K] extends Function ? T[K] : () => T[K]) : T[K];
};
export interface ComponentsConfig {
    [key: string]: any;
    ObjectWidget: ToDefault<ObjectWidgetProps>;
    StringWidget: ToDefault<StringWidgetProps>;
    TextAreaWidget: ToDefault<TextAreaWidgetProps>;
    PasswordWidget: ToDefault<PasswordWidgetProps>;
    ArrayWidget: ToDefault<ArrayWidgetContext>;
    NumberWidget: ToDefault<NumberWidgetProps>;
    DateWidget: ToDefault<DateWidgetProps>;
    SectionWidget: ToDefault<SectionWidgetProps>;
    FileWidget: ToDefault<FileWidgetProp>;
    CheckBoxWidget: ToDefault<CheckBoxWidgetProps>;
}

const config: ComponentsConfig = {} as ComponentsConfig;

export function useConfig<T extends keyof ComponentsConfig, K>(key: T, defaultValue?: K) {
    return deepAssign(defaultValue ?? {}, config?.[key] ?? {}) as K;
}

export function setConfig<T extends keyof ComponentsConfig>(key: T, value: ComponentsConfig[T]) {
    config[key] = value;
}
