import { deepAssign } from '@codeperate/utils';
import { defineAsyncComponent, markRaw } from 'vue';
import { FormSchema, IWidget } from '../FormBuilder.interface';
export function defineWidget<C, T, V>(widget: IWidget<T, V>) {
    return widget;
}

export const StringWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./StringWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const ObjectWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./ObjectWidget.vue'))),
    columns: 12,
    valueType: Object,
});

export const TextAreaWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./TextAreaWidget.vue'))),
    columns: 12,
    valueType: String,
});

export const PasswordWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./PasswordWidget.vue'))),
    columns: 12,
    valueType: String,
});

export const ArrayWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./ArrayWidget.vue'))),
    columns: 12,
    valueType: Array,
});

export const NumberWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./NumberWidget.vue'))),
    columns: 6,
    valueType: Number,
});

export const BooleanWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./BooleanWidget.vue'))),
    columns: 6,
    valueType: Boolean,
});

export const DateWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./DateWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const DateTimeWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./DateTimeWidget.vue'))),
    columns: 6,
    valueType: String,
});

export const SectionWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./SectionWidget.vue'))),
    columns: 12,
});
export const Section = (title: string, formSchema: Partial<FormSchema> = {}, key?: number | string | symbol) => {
    const obj = deepAssign({ widget: SectionWidget, props: { title }, label: false }, formSchema);
    return { [key ?? title]: obj };
};

export const FileWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./FileWidget.vue'))),
    columns: 12,
});

export const CheckBoxWidget = defineWidget({
    component: markRaw(defineAsyncComponent(() => import('./CheckBoxWidget.vue'))),
    columns: 12,
});
